import { Component, Vue, Watch } from 'vue-property-decorator';
import NavBar from '@/views/NavBar.vue';
import { logService, LogLevel, ApiLogAppender, ConsoleLogAppender } from '@/services/logging';
import { authStore } from '@/store';
import { User } from '@/models';

@Component({
  components: {
    NavBar
  }
})
export default class App extends Vue {

  private consoleLogAppender: ConsoleLogAppender;
  private apiLogAppender: ApiLogAppender;

  constructor() {
    super();

    // Setup application logging.
    this.consoleLogAppender = new ConsoleLogAppender(LogLevel.ALL);
    this.apiLogAppender = new ApiLogAppender(
      process.env.VUE_APP_PM_API_BASE + '/log/batch', '', LogLevel.WARN);
    logService.ROOT_LOGGER.appenders.push(this.apiLogAppender, this.consoleLogAppender);

    // TODO: prod/dev config settings for logging?
    this.apiLogAppender.batchSize = 1;
    this.apiLogAppender.minimumTimePassedInSec = 5;

    // Check for existing session cookie.
    /*tslint:disable:no-empty*/
    authStore.findLocalToken().catch(() => {});
  }

  private get authToken(): string | null {
    return authStore.authToken;
  }

  @Watch('authToken')
  private onAuthTokenChange(val: string | undefined , oldVal: string | undefined) {
    if (val) { this.apiLogAppender.authToken = val; }
  }
}
